/*
								+---------------------------------+
								|                                 |
								| ***  Fullscreen GDI Xample  *** |
								|                                 |
								|  Copyright  -tHE SWINe-  2002  |
								|                                 |
								|             Main.cpp            |
								|                                 |
								+---------------------------------+
*/

#include <windows.h>
#include "GDI_Interface.h"

static int nWidth, nHeight;
// vyska a sirka obrazovky

static unsigned __int32 *bmp_buffer;
// buffer pro bitmapu

static LPBITMAPINFO lpbi;
static HBITMAP hBitmap;
// informace pro GDI

// pripravi veci pro praci s GDI (graphix device interface)
int Init_GDI_Interface(int n_newWidth, int n_newHeight)
{
	nWidth = n_newWidth;
	nHeight = n_newHeight;
	// nastavi rozmery pro obrazovku

	lpbi = (LPBITMAPINFO) malloc(sizeof(BITMAPINFOHEADER) + (256 * sizeof(RGBQUAD)));
	if(lpbi == NULL)
		return 0;
	// naalokuje pamet pro bitmap info

	lpbi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	lpbi->bmiHeader.biWidth = nWidth;
	lpbi->bmiHeader.biHeight = nHeight;
	lpbi->bmiHeader.biPlanes = 1;
	lpbi->bmiHeader.biBitCount = 32;
	lpbi->bmiHeader.biCompression = BI_RGB;
	lpbi->bmiHeader.biSizeImage = nWidth * nHeight * 4;
	lpbi->bmiHeader.biXPelsPerMeter = 0;
	lpbi->bmiHeader.biYPelsPerMeter = 0;
	lpbi->bmiHeader.biClrUsed = 0;
	lpbi->bmiHeader.biClrImportant = 0;
	// nastavi bitmap info

	if((bmp_buffer = (unsigned __int32*) malloc(nWidth * nHeight * 4)) == NULL)
		return 0;
	// naalokuje buffer, kde se bodou uchovavat vlastni obrazova data

	hBitmap = CreateDIBSection(NULL, lpbi, DIB_RGB_COLORS, (void**)&bmp_buffer, NULL, 0);
	// vytvori handle pro Device Independent Bitmap (DIB)

	return 1;
}

// uvolni pamet zabranou bitmapama
void Terminate_GDI_Interface()
{
	free((void*)lpbi);
	//free((void*)bmp_buffer);
	// uvolni trosku pameti
}

// nakresli bitmapu bitmap (pole) o rozliseni
// nWidth  nHeight na zarizeni s handlem hdc
void Draw_Bitmap(HDC hdc, unsigned __int32 *bitmap)
{
	HDC hdc_bmp;

	hdc_bmp = CreateCompatibleDC(hdc); 
	SelectObject(hdc_bmp, hBitmap);
	// vytvori device kontext s nasi bitmapou

	SetDIBits(hdc_bmp, hBitmap, 0, nHeight, bitmap, lpbi, DIB_RGB_COLORS);
	// zkopiruje video ..

	StretchBlt(hdc, 0, 0, nWidth, nHeight, hdc_bmp, 0, nHeight, nWidth, -nHeight, SRCCOPY);
	// a kresli na hdc, musi se pouzit stretchblt() protoze SetDIBits() nam
	// prevraci obraz (sak je to nejake divne - ale trojhmat to sprasi :-))

	DeleteDC(hdc_bmp);
	// zase kontext smaze
}

// prepne do fullscreen
int Go_FullScreen()
{
	DEVMODE my_mode;
	int i;

	my_mode.dmFields = DM_BITSPERPEL + DM_PELSWIDTH + DM_PELSHEIGHT;
	// nastavi co vsechno se zmeni

	my_mode.dmSize = sizeof(DEVMODE);
	// musi byt nastaveno

	my_mode.dmPelsWidth = nWidth;
	my_mode.dmPelsHeight = nHeight;
	my_mode.dmBitsPerPel = 32;
	// nastavi nove vlastnosti

	if((i = ChangeDisplaySettings(&my_mode, 0)) != DISP_CHANGE_SUCCESSFUL) {
		my_mode.dmBitsPerPel = 16;
		// zkusi zmenit barvy quli pameti ve video

		if((i = ChangeDisplaySettings(&my_mode, 0)) != DISP_CHANGE_SUCCESSFUL)
			return i;
		// a pokud to nevyjde, konci
	}
	// zkusi zmenit mod

	/*
	  Navratove hodnoty :

	   suxess = 0
	   na restart = 1
	   failed = -1
	   badmode = -2
	   notupdated = -3
	   badflags = -4
	   badparam = -5
	*/

	ShowCursor(false);
	// schova mys

	return 1;
}

// prepne zpatky do normalniho rozliseni
void Back_FullScreen()
{
	ChangeDisplaySettings(NULL, 0);
	// tohle staci na zmenu rozliseni

	ShowCursor(true);
	// zase objevi mys
}