/*
								+---------------------------------+
								|                                 |
								|   ***   Blurred vectors   ***   |
								|                                 |
								|  Copyright  -tHE SWINe-  2002  |
								|                                 |
								|           Vectorz.cpp           |
								|                                 |
								+---------------------------------+
*/

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "Vectorz.h"

typedef struct {
	float x, y, z;
} Vector3f;
// vektor

typedef struct {
	Vector3f v[3];
} Matrix3v;
// matice bez pozice a spodniho radku

static int nWidth, nHeight;
// obrazovka

static unsigned char *buffer, *buffer_b;
// buffery pro obraz

static unsigned __int32 palette[256];
// paleta

static const int max_vectors = 2048;
static Vector3f vector[2048 * 5];
// vektory (5 ruznych sad mezi nimiz se interpoluje)

static int last_v_set;
static int cur_v_set;
static float last_time;
static float transform_time;
static float silence_time;
// predchozi a aktualni sada vektoru, posledni cas zmeny
// delka transformace, delka klidu

static float about_hold;
static int about_pos_x;
static int about_pos_y;
// cas zobrazeni loga, pozice x a y

#define PI ((float)3.141592653579893238462643383279502448)
// Ludolfovo cislo

// ----- Funkcni prototypy (privatni) -----------------------------------------------

Matrix3v rotX(float theta);
Matrix3v rotY(float theta);
Matrix3v rotZ(float theta);
Vector3f M3_Mult(Vector3f v, Matrix3v m);
Matrix3v M3_MtxMult(Matrix3v m, Matrix3v b);
// neco pro jednoduche prace s vektory, maticemi

void Blur();
// jednoduchy blur

void Zoom(int nCenterX, int nCenterY, float fAmount);
// zoom

void Draw_Line(int x1, int y1, int x2, int y2, unsigned char color, unsigned char *video);
// kresli caru

void Draw_About_Logo(int nCenterX, int nCenterY);
// kresli logo



// ----- Funkce ---------------------------------------------------------------------

// nastavi vektory, naalokuje blurovaci buffery
int Vectorz_Init(int n_newWidth, int n_newHeight)
{
	Vector3f vec;
	Matrix3v mtx;
	int i;

	nWidth = n_newWidth;
	nHeight = n_newHeight;
	// nastavi rozmery pro obrazovku

	if((buffer = (unsigned char*) malloc(nWidth * nHeight)) == NULL)
		return 0;
	if((buffer_b = (unsigned char*) malloc(nWidth * nHeight)) == NULL)
		return 0;
	// naalokuje buffery

	for(i = 0; i < max_vectors; i ++) {
		mtx = rotX(2 * PI * (float)sin((float)i / 203));
		vec.x = 64 + 16 * (float)sin((float)i / 191);
		vec.y = 0;
		vec.z = 0;
		vec = M3_Mult(vec, mtx);
		mtx = rotY(2 * PI * (float)cos((float)i / 157));
		vec = M3_Mult(vec, mtx);
		mtx = rotZ(-2 * PI * (float)cos((float)i / 181));
		vector[i + max_vectors] = M3_Mult(vec, mtx);
	}
	// spocita zvlastni chuchvalec

	for(i = 0; i < max_vectors; i ++) {
		vec.x = 80;
		vec.y = 0;
		vec.z = 0;
		mtx = rotY((2 * PI / 5 * 2) * i);
		vector[i + max_vectors * 2] = M3_Mult(vec, mtx);
	}
	// spocita pentagram

	for(i = 0; i < max_vectors; i ++) {
		vec.x = 80;
		vec.y = 0;
		vec.z = 0;
		mtx = rotY(2 * PI * i / (max_vectors - 1));
		vec = M3_Mult(vec, mtx);
		mtx = rotZ(2 * PI * i / ((max_vectors - 1) / 8));
		vector[i + max_vectors * 3] = M3_Mult(vec, mtx);
	}
	// spocita spiralu (8 je pocet zavitu)

	for(i = 0; i < max_vectors; i ++) {
		vec.x = 80;
		vec.y = 0;
		vec.z = 0;
		mtx = rotY((2 * PI / 5 * 2) * i);
		vec = M3_Mult(vec, mtx);
		mtx = rotZ((2 * PI / 9 * 4) * i);
		vector[i + max_vectors * 4] = M3_Mult(vec, mtx);
	}
	// spocita "jezka"

	memset(buffer, 0, nWidth * nHeight);
	memset(buffer_b, 0, nWidth * nHeight);
	// vynuluje oba buffery

	srand(time(NULL));
	// generator nahod. cisel

	last_v_set = (rand() & 3) + 1;
	cur_v_set = (rand() & 3) + 1;
	// vybere nahodne objekty

	for(i = 0; i < max_vectors; i ++)
		vector[i] = vector[i + max_vectors * cur_v_set];
	// zkopiruje aktualni sadu vektoru

	last_time = (float)clock() / 40;
	transform_time = 60 + (float)(rand() % 80);
	silence_time = (rand() % 60) + last_time;
	// nastavi casy

	about_hold = 0;
	// vypne logo

	return 1;
}

// uvloni nejakou pamet
void Vectors_Terminate()
{
	free((void*)buffer);
	free((void*)buffer_b);
}

// interpoluje vektor
Vector3f inline Interpolate(Vector3f a, Vector3f b, float t)
{
	Vector3f v;
	float s;

	s = 1 - t;
	v.x = a.x * t + b.x * s;
	v.y = a.y * t + b.y * s;
	v.z = a.z * t + b.z * s;

	return v;
}

// kresli chuchvalec
void Vectorz_Draw(unsigned __int32 *video)
{
	float iz, x, y, b_dist;
	Matrix3v mtx;
	Vector3f vec;
	float fFlash;
	int r, g, b;
	int sx, sy;
	int px, py;
	float t;
	int i;

    t = (float)clock() / 40;
	// aktualni cas

	Zoom((int)(nWidth / 2 + (nWidth / 4) * (float)sin(t / 219)),
		 (int)(nHeight / 2 + (nHeight / 4) * (float)sin(t / 159)), (float)0.05);
	// zvetsi obraz

	for(i = 0; i < max_vectors; i += 4) {
		if(t > silence_time) {
			if(t - silence_time > transform_time) {
				last_v_set = cur_v_set;
				cur_v_set = (rand() & 3) + 1;
				// vybere nove nahodne objekty

				last_time = t;
				transform_time = 60 + (float)(rand() % 80);
				silence_time = (rand() % 60) + last_time;
				// nastavi nove casy
			}
			vector[i] = Interpolate(vector[i + cur_v_set * max_vectors],
				vector[i + last_v_set * max_vectors], (t - silence_time) / transform_time);
		}

		b_dist = 256 + 64 * (float)sin(t / 327);
        mtx = M3_MtxMult(rotX(2 * PI * (float)sin(t / 289)),
						 rotY(2 * PI * (float)cos(t / 307)));
        mtx = M3_MtxMult(mtx, rotZ(-2 * PI * (float)sin(t / 251)));
        vec = M3_Mult(vector[i], mtx);
		// transformace, deformace...

		iz = 1 / (vec.z + b_dist);
		x = nWidth / 2 + (nWidth / 2) * vec.x * iz;
		y = nHeight / 2 + (nHeight / 2) * vec.y * iz;
		// zpetna transformace (chuchvalec je uprostred, 200 pixelu zpatky od obrazovky)

		if(x < 0 || x >= nWidth || y < 0 || y >= nHeight)
			continue;
		// clipping

		sx = (int)(x + 0.5);
		sy = (int)(y + 0.5);
		// integer pozice

		//buffer[sx + nWidth * sy] = 0xff;
		// kresli vektor

		if(i)
			Draw_Line(sx, sy, px, py, 0xff, buffer);
		// kresli caru mezi vektory

		px = sx;
		py = sy;
		// predchozi souradnice
	}

	if(rand() < 250) {
		about_hold = t + 160; // 4 sec.
		about_pos_x = rand() % (nWidth - 130) + 65;
		about_pos_y = rand() % (nHeight - 80) + 40;
	}
	// obcas vygeneruje novou pozici loga

	if(about_hold > t)
		Draw_About_Logo(about_pos_x, about_pos_y);
	// obcas vykresli logo (velikost loga je 130 x 80)

	Blur();
	// vytvori blur

	fFlash = 0;
	// flashovani - tady muze byt taky nakej sinus
	// (hodnoty 0 - 1) - treba synchro s hudbou ..

	r = (int)(128 + 127 * cos(t / 74)),
    g = (int)(128 + 127 * cos(t / 63)),
    b = (int)(128 + 127 * cos(t / 81));
    for(i = 0; i < 128; i ++) {
        palette[i] = ((int)((r * i / 128) * (1 - fFlash) + 255 * fFlash) << 16) +
                     ((int)((g * i / 128) * (1 - fFlash) + 255 * fFlash) << 8) +
                      (int)((b * i / 128) * (1 - fFlash) + 255 * fFlash);
        palette[i + 128] = ((int)((r + (255 - r) * i / 128) * (1 - fFlash) + 255 * fFlash) << 16) +
                           ((int)((g + (255 - g) * i / 128) * (1 - fFlash) + 255 * fFlash) << 8) +
                            (int)((b + (255 - b) * i / 128) * (1 - fFlash) + 255 * fFlash);
    }
	// vytvori paletu (prechod >> 128 = [r, g, b], 0 = cerna, 255 = bila)

	for(i = 0; i < nWidth * nHeight; i ++)
		video[i] = palette[buffer[i]];
	// vytvori obraz
}




// ----- Funkce (privatni) ----------------------------------------------------------

// kresli caru
void Draw_Line(int x1, int y1, int x2, int y2, unsigned char color, unsigned char *video)
{
	double length;
	float deltax;
	float deltay;
	float xa, ya;
	int i, x, y;
	double hl;
	double vl;

	hl = fabs((float)(x2 - x1));
	vl = fabs((float)(y2 - y1));
	length = (hl > vl) ? hl : vl;
	deltax = (float)(x2 - x1) / (float)length;
	deltay = (float)(y2 - y1) / (float)length;
	xa = (float)x1;
	ya = (float)y1;

	for(i = 0; i < (int)length; i ++) {
		x = (int)(xa += deltax);
		y = (int)(ya += deltay);
		
		if(x >= 0 && x < nWidth && y >= 0 && y < nHeight)
			video[x + y * nWidth] = color;
	}
}

// rozmazne obraz v buffer (buffer_b je jen vyrovnavaci pamet)
void Blur()
{
	unsigned char b;
	int x, sy, y;

	for(sy = 1; sy < nHeight - 1; sy ++) {
		y = sy * nWidth;
		for(x = 1; x < nWidth - 1; x ++) {
			b = (buffer[x - 1 + y - nWidth] +
			     buffer[x - 1 + y + nWidth] +
			     buffer[x     + y - nWidth] +
			     buffer[x     + y + nWidth] +
			     buffer[x - 1 + y         ] +
			     buffer[x     + y         ] +
			     buffer[x + 1 + y         ] +
			     buffer[x + 1 + y - nWidth] +
			     buffer[x + 1 + y + nWidth]) / 9;
			// spocita prumer
			
			b = (b > 16)? b - 16 : 0;
			// ztmavi obraz

			buffer_b[x + y] = b;
			// ulozi hodnotu
		}
	}
	// rozmazne obraz

	memcpy(buffer, buffer_b, nWidth * nHeight);
	// zkopiruje buffer zpatky
}

// vyzoomuje buffer (buffer_b je zase vyrovnavaci pamet)
void Zoom(int nCenterX, int nCenterY, float fAmount)
{
	int sy, scale_x, scale_y;
	int x, y;

	fAmount = 1 - fAmount;

	for(sy = 0; sy < nHeight; sy ++) {
		scale_y = (int)(nCenterY + (sy - nCenterY) * fAmount) * nWidth;
		y = sy * nWidth;
		for(x = 0; x < nWidth; x ++) {
			scale_x = (int)(nCenterX + (x - nCenterX) * fAmount);
			//
			buffer_b[x + y] = buffer[scale_x + scale_y];
		}
	}
	// zvetsi obrazek

	memcpy(buffer, buffer_b, nWidth * nHeight);
	// zkopiruje buffer zpatky
}

// kresli logo (nahodne)
void Draw_About_Logo(int nCenterX, int nCenterY)
{
	float lines[][4] = {
		{-3, -8, -3, -4},
		{-3, -8, -1, -7},
		{-1, -7, -3, -6},
		{-3, -6, -1, -5},
		{-1, -5, -3, -4}, //B
		{0, -8, 0, -4}, // l
		{1, -6, 1, -4},
		{1, -4, 2, -4},
		{2, -4, 2, -6}, // u
		{3, -6, 3, -4},
		{3, -6, 4, -6}, // r
		{-1, -2, -1, 2},
		{-1, 0, 0, 1},
		{0, 1, -1, 2}, // b
		{1, 0, 1, 1},
		{1, 1, 2, 1},
		{2, 0, 2, 2},
		{2, 2, 1, 2}, // y
		{-13, 6, -12, 6}, // -
		{-11, 4, -11, 8},
		{-11, 5, -10, 5},
		{-11, 8, -10, 8}, // t
		{-9, 4, -9, 8},
		{-9, 6, -7, 6},
		{-7, 4, -7, 8}, // H
		{-6, 4, -6, 8},
		{-6, 4, -4, 4},
		{-6, 6, -5, 6},
		{-6, 8, -4, 8}, // E
		{-2, 8, 0, 8},
		{0, 8, 0, 6},
		{0, 6, -2, 6},
		{-2, 6, -2, 4},
		{-2, 4, 0, 4}, // S
		{1, 4, 1, 8},
		{1, 8, 3, 8},
		{3, 8, 3, 4},
		{2, 8, 2, 6}, // W
		{4, 8, 6, 8},
		{4, 4, 6, 4},
		{5, 4, 5, 8}, // I
		{7, 8, 7, 4},
		{7, 4, 9, 8},
		{9, 8, 9, 4}, // N
		{10, 7, 12, 7},
		{12, 7, 12, 5},
		{12, 5, 10, 5},
		{10, 5, 10, 8},
		{10, 8, 12, 8}, // e
		{13, 6, 14, 6}}; // -
	int i, n;

	n = sizeof(lines) / (4 * sizeof(int));
	// pocet car

	for(i = 0; i < n; i ++) {
		Draw_Line(nCenterX + (int)(5 * lines[i][0]),
				  nCenterY + (int)(5 * lines[i][1]),
				  nCenterX + (int)(5 * lines[i][2]),
				  nCenterY + (int)(5 * lines[i][3]), 0xff, buffer);
	}
	// kresli cary
}

// vynasobi matici matici
Matrix3v M3_MtxMult(Matrix3v m, Matrix3v b)
{
	Matrix3v r;
	int i;

	for(i = 0; i < 3; i ++) {
		r.v[i].x = m.v[i].x * b.v[0].x +
			       m.v[i].y * b.v[1].x +
			       m.v[i].z * b.v[2].x;
		r.v[i].y = m.v[i].x * b.v[0].y +
			       m.v[i].y * b.v[1].y +
			       m.v[i].z * b.v[2].y;
		r.v[i].z = m.v[i].x * b.v[0].z +
			       m.v[i].y * b.v[1].z +
			       m.v[i].z * b.v[2].z;
	}

	return r;
}

// vynasobi vektor matici (transformace bez posunuti - jen rotace)
Vector3f M3_Mult(Vector3f v, Matrix3v m)
{
	Vector3f tmp;

	tmp.x = v.x * m.v[0].x + v.y * m.v[1].x + v.z * m.v[2].x;
	tmp.y = v.x * m.v[0].y + v.y * m.v[1].y + v.z * m.v[2].y;
	tmp.z = v.x * m.v[0].z + v.y * m.v[1].z + v.z * m.v[2].z;

	return tmp;
}

// spocita rotacni matici po ose x
Matrix3v rotX(float theta)
{
	Matrix3v tmp;
	float c, s;

	c = (float)cos(theta);
	s = (float)sin(theta);

	tmp.v[0].x = 1;
	tmp.v[0].y = 0;
	tmp.v[0].z = 0;
	tmp.v[1].x = 0;
	tmp.v[1].y = c;
	tmp.v[1].z = -s;
	tmp.v[2].x = 0;
	tmp.v[2].y = s;
	tmp.v[2].z = c;

	return tmp;
}

// spocita rotacni matici po ose y
Matrix3v rotY(float theta)
{
	Matrix3v tmp;
	float c, s;

	c = (float)cos(theta);
	s = (float)sin(theta);

	tmp.v[0].x = c;
	tmp.v[0].y = 0;
	tmp.v[0].z = s;
	tmp.v[1].x = 0;
	tmp.v[1].y = 1;
	tmp.v[1].z = 0;
	tmp.v[2].x = -s;
	tmp.v[2].y = 0;
	tmp.v[2].z = c;

	return tmp;
}

// spocita rotacni matici po ose z
Matrix3v rotZ(float theta)
{
	Matrix3v tmp;
	float c, s;

	c = (float)cos(theta);
	s = (float)sin(theta);

	tmp.v[0].x = c;
	tmp.v[0].y = -s;
	tmp.v[0].z = 0;
	tmp.v[1].x = s;
	tmp.v[1].y = c;
	tmp.v[1].z = 0;
	tmp.v[2].x = 0;
	tmp.v[2].y = 0;
	tmp.v[2].z = 1;

	return tmp;
}